'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (content) {
	this.cacheable && this.cacheable();

	const callback = this.async();
	var _options = this.options;
	const context = _options.context,
	      output = _options.output,
	      _module = this._module;

	const options = (0, _loaderUtils.getOptions)(this) || {};

	const hasIssuer = _module && _module.issuer;
	const issuerContext = hasIssuer && _module.issuer.context || context;

	var _options$root = options.root;
	const root = _options$root === undefined ? (0, _path.resolve)(context, issuerContext) : _options$root;
	var _options$publicPath = options.publicPath;

	const publicPath = _options$publicPath === undefined ? output.publicPath || '/' : _options$publicPath,
	      forceMinimize = options.minimize,
	      minimizeOptions = _objectWithoutProperties(options, ['root', 'publicPath', 'minimize']);

	const requests = [];
	const hasMinimzeConfig = typeof forceMinimize === 'boolean';
	const shouldMinimize = hasMinimzeConfig ? forceMinimize : this.minimize;

	const minimize = content => new _minimize2.default(_extends({}, defaultMinimizeConf, minimizeOptions)).parse(content);

	const loadModule = request => new Promise((resolve, reject) => {
		this.loadModule(request, (err, src) => {
			if (err) {
				reject(err);
			} else {
				resolve(src);
			}
		});
	});

	const replace = (() => {
		var _ref2 = _asyncToGenerator(function* (_ref) {
			let request = _ref.request,
			    startIndex = _ref.startIndex,
			    endIndex = _ref.endIndex;

			const src = yield loadModule(request);
			const replacement = extract(src, publicPath);
			content = replaceAt(content, startIndex, endIndex, replacement);
		});

		return function replace(_x) {
			return _ref2.apply(this, arguments);
		};
	})();

	const parser = _sax2.default.parser(false, { lowercase: true });

	parser.onattribute = (_ref3) => {
		let name = _ref3.name;
		var _ref3$value = _ref3.value;
		let value = _ref3$value === undefined ? '' : _ref3$value;

		if (!isSrc(name) || !(0, _loaderUtils.isUrlRequest)(value, root)) {
			return;
		}

		const endIndex = parser.position - 1;
		const startIndex = endIndex - value.length;
		const request = (0, _loaderUtils.urlToRequest)(value, root);
		requests.unshift({ request, startIndex, endIndex });
	};

	parser.onend = _asyncToGenerator(function* () {
		yield Promise.all(requests.map(replace));
		if (shouldMinimize) {
			content = minimize(content);
		}
		callback(null, content);
	});

	parser.write(content).close();
};

var _path = require('path');

var _sax = require('sax');

var _sax2 = _interopRequireDefault(_sax);

var _vm = require('vm');

var _minimize = require('minimize');

var _minimize2 = _interopRequireDefault(_minimize);

var _loaderUtils = require('loader-utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const isSrc = name => name === 'src';
const replaceAt = (str, start, end, replacement) => str.slice(0, start) + replacement + str.slice(end);

const extract = (src, __webpack_public_path__) => {
	const script = new _vm.Script(src, { displayErrors: true });
	const sandbox = {
		__webpack_public_path__,
		module: {}
	};
	script.runInNewContext(sandbox);
	return sandbox.module.exports.toString();
};

const defaultMinimizeConf = {
	empty: true,
	quotes: true,
	dom: { recognizeSelfClosing: true }
};